# Install these R packages before you test the code
# library("mblm")
# library("Hmisc")
# library("energy")
# library("mvtnorm")
# library("pcaPP")
# library("rrcov")
# library("robustbase")
# library("biwt")
# library(stats)

#two functions required

cor.pear<-function(x,y)
{

   r<-cor(x,y)
  n<-length(x)
T<-r*sqrt(n-2)/sqrt(1-r^2)
pvalue<-2*pt(-abs(T),df=n-2)
return(list(pvalue,T))
}

wt.rank.corr<-function(x,y)
  { n<-length(x)
    
 w<-(x-y)^2*((n-x+1)+(n-y+1))

  rw<-1-6/(n^4+n^3-n^2-n)*(sum(w))

var.rw =(31*n^2+60*n+26)/(30*(n^3+n^2-n-1))

Z<-rw/sqrt(var.rw)
pvalue<-2*pnorm(-abs(Z))
return(list(pvalue,Z))
}

# read your data. Make sure you have the correct path to read the data
# you perform gene association between two sets of genes

genes<-as.matrix(read.table("./sample_data/genes_of_interest_expdata.txt"))
data<-as.matrix(read.table("./sample_data/data_of_other_genes.txt"))

#some parameters:
#cutoff="notrue"
cutoff="true"
sig<-(10)^(-7)
res_method_1=NULL
 for(j in 1:3)
{    
y<-genes[j,]
y<-as.numeric(y[-1])
y1<-rank(y)
m<-dim(data)[1]


# you can choose some of eight methods to use

## first method----theil-sen----(very slow)--------------------------------------------------------
pvalue<-c(1:m)
for (i in 1:m)
      {
    x<-as.numeric(data[i,][-1])
         fit<-mblm(y~x)
 
   pvalue[i]<-summary(fit)$coef[8]
}
gene.names<-data[,1]
M<-cbind(gene.names,pvalue)
method1<-M[order(as.numeric(M[,2])),]
if(cutoff=="notrue"){
method_1<-method1}
if(cutoff=="true")
{method_1<-method1[which(as.numeric(method1[,2])<sig,]}
res_method_1<-rbind(res_method_1,method_1)
   
write.table(res_method_1, file="/data/home/skumari/method.theil.sen.txt", row.names=F, col.names=c("gene.names","pvalues"), quote=F, sep="\t")


#-------------------------
res_method_2<-NULL
for(j in 1:3)
{    
y<-genes[j,]
y<-as.numeric(y[-1])
y1<-rank(y)
m<-dim(data)[1]

##-Second method-----------pearson--------------------------------------------------------

pvalue1<-c(1:m)
for (i in 1:m)
      {
      x<-as.numeric(data[i,][-1])
       
  pvalue1[i]<-cor.pear(x,y)[[1]]
}
gene.names1<-data[,1]
M1<-cbind(gene.names1,pvalue1)
method2<-M1[order(as.numeric(M1[,2])),]
if(cutoff=="notrue"){
method_2<-method2}
if(cutoff=="true")
{method_2<-method2[which(as.numeric(method2[,2])<sig),]}
res_method_2<-rbind(res_method_2,method_2)
}
write.table(res_method_2, file="/data/home/skumari/method.pearson.txt", row.names=F, col.names=c("gene.names","pvalues"), quote=F, sep="\t")




##-------------------------hoeffd--------------
res_method_3<-NULL
for(j in 1:3)
{    
y<-genes[j,]
y<-as.numeric(y[-1])
y1<-rank(y)
m<-dim(data)[1]

pvalue2<-c(1:m)
dstat<-c(1:m)
for (i in 1:m)
      {
      x<-as.numeric(data[i,][-1])
        fit3<-hoeffd(x,y)
 
   pvalue2[i]<-fit3$P[2]
 dstat[i]<-fit3$D[2]
}

gene.names2<-data[,1]
M2<-cbind(gene.names2,pvalue2,dstat)
method3<-M2[order(as.numeric(M2[,2]),-as.numeric(M2[,3])),]
if(cutoff=="notrue"){
method_3<-method3}
if(cutoff=="true")
{method_3<-method3[which(as.numeric(method3[,2])<sig),]}
res_method_3<-rbind(res_method_3,method_3)
}
write.table(res_method_3, file="/data/home/skumari/method.hoeffd.txt", row.names=F, col.names=c("gene.names","pvalues","stat"), quote=F, sep="\t")


##------------------------spearman-----------------------------------
res_method_4<-NULL
for(j in 1:3)
{    
y<-genes[j,]
y<-as.numeric(y[-1])
y1<-rank(y)
m<-dim(data)[1]
pvalue3<-c(1:m)
for (i in 1:m)
      {
      x<-as.numeric(data[i,][-1])
         fit4<-cor.test(x,y, method="spearman", alternative="two.sided",exact=FALSE)
 
   pvalue3[i]<-fit4$p.value
}
gene.names3<-data[,1]
M4<-cbind(gene.names3,pvalue3)
method4<-M4[order(as.numeric(M4[,2])),]
if(cutoff=="notrue"){
method_4<-method4}
if(cutoff=="true")
{method_4<-method4[which(as.numeric(method4[,2])<sig),]}
res_method_4<-rbind(res_method_4,method_4)
}
write.table(res_method_4, file="/data/home/skumari/method.spearman.txt", row.names=F, col.names=c("gene.names","pvalues"), quote=F, sep="\t")


##-------------------------rank theil-sen------------(slow methods)---------------
res_method_5<-NULL
for(j in 1:3)
{    
y<-genes[j,]
y<-as.numeric(y[-1])
y1<-rank(y)
m<-dim(data)[1]


pvalue4<-c(1:m)
stats<-c(1:m)
for (i in 1:m)
   {
    x<-as.numeric(data[i,][-1])
    x<-rank(x)
         fit<-mblm(y1~x)
 
   pvalue4[i]<-summary(fit)$coef[8]

}

gene.names4<-data[,1]
M5<-cbind(gene.names4,pvalue4)
method5<-M5[order(as.numeric(M5[,2])),]
if(cutoff=="notrue"){
method_5<-method5}
if(cutoff=="true")
{method_5<-method5[which(as.numeric(method5[,2])<sig),]}
res_method_5<-rbind(res_method_5,method_5)
}
write.table(res_method_5, file="/data/home/skumari/method.rank.theil.txt", row.names=F, col.names=c("gene.names","pvalues"), quote=F, sep="\t")


###--------------------------------dist covariance-----------------------------------
res_method_6<-NULL
for(j in 1:3)
{    
y<-genes[j,]
y<-as.numeric(y[-1])
y1<-rank(y)
m<-dim(data)[1]

pvalue5<-c(1:m)
stats1<-c(1:m)
for (i in 1:m)
      {
      x<-as.numeric(data[i,][-1])
         fit1<-dcov.test(x,y)
 
   pvalue5[i]<-fit1$p.value
stats1[i]<-fit1$statistic
}
gene.names5<-data[,1]
M6<-cbind(gene.names5,pvalue5,stats1)
method6<-M6[order(as.numeric(M6[,2]),-as.numeric(M6[,3])),]
if(cutoff=="notrue"){
method_6<-method6}
if(cutoff=="true")
{method_6<-method6[which(as.numeric(method6[,2])<sig),]}
res_method_6<-rbind(res_method_6,method_6)
}
write.table(res_method_6, file="/data/home/skumari/method.dist.corr.txt", row.names=F, col.names=c("gene.names","pvalues","stat"), quote=F, sep="\t")



#####--------------------------kendall------------------------
res_method_7<-NULL
for(j in 1:3)
{    
y<-genes[j,]
y<-as.numeric(y[-1])
y1<-rank(y)
m<-dim(data)[1]

pvalue6<-c(1:m)
stats2<-c(1:m)
for (i in 1:m)
      {
      x<-as.numeric(data[i,][-1])
         fit2<-cor.test(x,y, method="kendall", alternative="two.sided",exact=FALSE)
 
   pvalue6[i]<-fit2$p.value
stats2[i]<-fit2$statistic
}

gene.names6<-data[,1]
M7<-cbind(gene.names6,pvalue6,stats2)
method7<-M7[order(as.numeric(M7[,2]),-as.numeric(M7[,3])),]
if(cutoff=="notrue"){
method_7<-method7}
if(cutoff=="true")
{method_7<-method7[which(as.numeric(method7[,2])<sig),]}
res_method_7<-rbind(res_method_7,method_7)
}
write.table(res_method_7, file="/data/home/skumari/method.kendall.txt", row.names=F, col.names=c("gene.names","pvalues","stat"), quote=F, sep="\t")



###-------------------------------wt.rank.corr--------------------------------
res_method_8<-NULL
for(j in 1:3)
{    
y<-genes[j,]
y<-as.numeric(y[-1])
y1<-rank(y)
m<-dim(data)[1]

pvalue7<-c(1:m)
for (i in 1:m)
      {
    x<-as.numeric(data[i,][-1])
     x<-rank(x)
         fit3<-wt.rank.corr(x,y1)
 pvalue7[i]<-fit3[[1]]
 stats3[i]<-fit3[[2]]

}

gene.names7<-data[,1]
M8<-cbind(gene.names7,pvalue7)
method8<-M8[order(as.numeric(M8[,2])),]
if(cutoff=="notrue"){
method_8<-method8}
if(cutoff=="true")
{method_8<-method8[which(as.numeric(method8[,2])<sig),]}
res_method_8<-rbind(res_method_8,method_8)
}
write.table(res_method_8, file="/data/home/skumari/method.wt.rank.corr.txt", row.names=F, col.names=c("gene.names","pvalues"), quote=F, sep="\t")




